package uoc.tfg.g2tde;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import uoc.tfg.g2tde.entities.Rol;
import uoc.tfg.g2tde.entities.Usuario;
import uoc.tfg.g2tde.interfaces.IUsuarioService;

@Service("jpaUserDetailsService")
public class JpaUserDetailsService implements UserDetailsService {

	@Autowired
	private IUsuarioService usuarioService;

	@Override
	public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

		Usuario usuario = usuarioService.findByUsername(username);
		if (usuario == null) {
			throw new UsernameNotFoundException("El usuario: " + username + " no existe en el sistema!");
		}
		
		List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
		
		Set<Rol> roles = usuario.getRoles();
		if(usuario.getRoles() != null) {
			authorities = roles.stream()
				.map(r->new SimpleGrantedAuthority(r.getNombre()))
				.collect(Collectors.toList());
		}
		
		if (authorities.isEmpty()) {
			throw new UsernameNotFoundException("El usuario '" + username + "' no tiene roles");
		}
		return new User(usuario.getUsername(), usuario.getPassword(), true, true, true, true, authorities);
	}

}
