package uoc.tfg.g2tde.controllers;

import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Conclusion;
import uoc.tfg.g2tde.interfaces.IConclusionService;
import uoc.tfg.g2tde.interfaces.IReglaService;
import uoc.tfg.g2tde.interfaces.IUsuarioService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/conclusion")
@SessionAttributes("conclusion")
@Secured({"ROLE_OPERADOR"})
public class ConclusionController {

	@Autowired
	private IConclusionService conclusionService;
	
	@Autowired
	private IUsuarioService usuarioService;
	
	@Autowired
	private IReglaService reglaService;
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Conclusiones");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Conclusion> conclusiones = conclusionService.findAll(pageRequest);
		PageRender<Conclusion> pageRender = new PageRender<Conclusion>("/conclusion/listar", conclusiones);
		model.addAttribute("title", "Listado de conclusiones");
		model.addAttribute("conclusiones", conclusiones);
		model.addAttribute("page", pageRender);
		
		return "/conclusion/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Conclusion> conclusion = conclusionService.findById(id);
		if(!conclusion.isPresent()) {
			flash.addFlashAttribute("error", "No existe la conclusion");
			return "redirect:/conclusion/listar";
		}

		model.put("conclusion", conclusion.get());
		model.put("title", "Detalle conclusión");
		return "/conclusion/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		Conclusion conclusion = new Conclusion();
		model.put("conclusion", conclusion);
		model.put("title", "Crear conclusión");
		return "/conclusion/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		
		Optional<Conclusion> conclusion = null;
		
		if(id > 0) {
			conclusion = conclusionService.findById(id);
		} else {
			return "redirect:/listar";
		}
		model.put("conclusion", conclusion);
		model.put("title", "Editar Conclusion");
		return "/conclusion/form";
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Conclusion conclusion, BindingResult result, Model model, SessionStatus status) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Conclusion");
			return "/conclusion/form";
		}
		
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		conclusion.setUsuario(usuarioService.findByUsername(authentication.getName()));
		
		conclusion.setEsCalculada("N");
		
		conclusionService.save(conclusion);
		status.setComplete();
		return "redirect:/conclusion/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
		
		Optional<Conclusion> conclusion = conclusionService.findById(id);
		
		conclusion.ifPresent(c->{
			reglaService.findByConclusionId(c.getId()).stream().forEach(r->reglaService.deleteById(r.getId()));
			
			conclusionService.deleteById(c.getId());
		});
		return "redirect:/conclusion/listar";
	}
}
