package uoc.tfg.g2tde.controllers;

import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Consultoria;
import uoc.tfg.g2tde.interfaces.IConsultoriaService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/consultoria")
@SessionAttributes("consultoria")
@Secured({"ROLE_GESTION"})
public class ConsultoriaController {

	@Autowired
	private IConsultoriaService consultoriaService;
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de consultorías");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Consultoria> consultorias = consultoriaService.findAll(pageRequest);
		PageRender<Consultoria> pageRender = new PageRender<Consultoria>("/consultoria/listar", consultorias);
		model.addAttribute("title", "Listado de consultorías");
		model.addAttribute("consultorias", consultorias);
		model.addAttribute("page", pageRender);
		
		return "/consultoria/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Consultoria> consultoria = consultoriaService.findById(id);
		if(!consultoria.isPresent()) {
			flash.addFlashAttribute("error", "No existe la consultoría");
			return "redirect:/consultoria/listar";
		}

		model.put("consultoria", consultoria.get());
		model.put("title", "Detalle consultoria");
		return "/consultoria/detalle";
	}
}
