package uoc.tfg.g2tde.controllers;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Consultoria;
import uoc.tfg.g2tde.entities.Empleado;
import uoc.tfg.g2tde.entities.EmpleadoClienteTarea;
import uoc.tfg.g2tde.interfaces.IConsultoriaService;
import uoc.tfg.g2tde.interfaces.IEmpleadoClienteTareaService;
import uoc.tfg.g2tde.interfaces.IEmpleadoService;
import uoc.tfg.g2tde.interfaces.ITareaService;
import uoc.tfg.g2tde.services.UsuarioService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/empleado")
@SessionAttributes("empleado")
@Secured({"ROLE_GESTION"})
public class EmpleadoController {

	@Autowired
	private IEmpleadoService empleadoService;
	
	@Autowired
	private IConsultoriaService consultoriaService;
	
	@Autowired
	private IEmpleadoClienteTareaService empleadoClienteTareaService;
	
	@Autowired
	private ITareaService tareaService;
	
	@Autowired
	private UsuarioService usuarioService;
	
	@ModelAttribute("consultorias")
	public List<Consultoria> listaConsultorias(){
		return consultoriaService.findAll();
	}

	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Empleados");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Empleado> empleados = empleadoService.findAll(pageRequest);
		PageRender<Empleado> pageRender = new PageRender<Empleado>("/empleado/listar", empleados);
		model.addAttribute("title", "Listado de empleados");
		model.addAttribute("empleados", empleados);
		model.addAttribute("page", pageRender);
		
		return "/empleado/listar";
	}

	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {
		Optional<Empleado> empleado = empleadoService.findById(id);
		if (!empleado.isPresent()) {
			flash.addFlashAttribute("error", "No existe el criterio");
			return "redirect:/empleado/listar";
		}
		model.put("empleado", empleado.get());
		model.put("title", "Detalle empleado");
		return "/empleado/detalle";
	}

	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {
		Empleado empleado = new Empleado();
		model.put("empleado", empleado);
		model.put("title", "Crear empleado");
		return "/empleado/form";
	}

	@RequestMapping(value = "/form/{id}")
	public String editar(@PathVariable(value = "id") Long id, Map<String, Object> model) {
		Optional<Empleado> empleadoOpt = empleadoService.findById(id);
		if (empleadoOpt.isPresent()) {
			Empleado empleado = empleadoOpt.get();
			model.put("empleado", empleado);
			model.put("title", "Editar Empleado");
			return "/empleado/form";
		} else {
			return "redirect:/empleado/listar";
		}
	}

	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Empleado empleado, BindingResult result, Model model, SessionStatus status) {
		if (result.hasErrors()) {
			model.addAttribute("title", "Formulario de Empleado");
			return "/empleado/form";
		}
		empleadoService.save(empleado);
		status.setComplete();
		return "redirect:/empleado/listar";
	}

	@RequestMapping(value = "/eliminar/{id}")
	public String eliminar(@PathVariable(value = "id") Long id) {
		Optional<Empleado> empleadoOpt = empleadoService.findById(id);
		if (empleadoOpt.isPresent()) {
			
			if(empleadoOpt.get().getUsuario() != null) {
				usuarioService.deleteById(empleadoOpt.get().getUsuario().getUsername());
			}
			
			List<EmpleadoClienteTarea> empleadoClienteTareaList = empleadoClienteTareaService.findByClienteId(id);
			
			empleadoService.deleteById(id);
			empleadoClienteTareaList.stream().forEach(e->tareaService.deleteById(e.getTarea().getId()));
		}
		return "redirect:/empleado/listar";
	}
}
