package uoc.tfg.g2tde.controllers;

import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Informe;
import uoc.tfg.g2tde.interfaces.IInformeService;
import uoc.tfg.g2tde.interfaces.IUsuarioService;
import uoc.tfg.g2tde.ssee.interfaces.ISistemaExpertoService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/informe")
@SessionAttributes("informe")
@Secured({"ROLE_OPERADOR"})
public class InformeController {

	@Autowired
	private IInformeService informeService;
	
	@Autowired
	private IUsuarioService usuarioService;
	
	@Autowired
	private ISistemaExpertoService sistemaExpertoService;
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Informe");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Informe> informes = informeService.findAll(pageRequest);
		PageRender<Informe> pageRender = new PageRender<Informe>("/informe/listar", informes);
		model.addAttribute("title", "Listado de informes");
		model.addAttribute("informes", informes);
		model.addAttribute("page", pageRender);
		
		return "/informe/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Informe> informe = informeService.findById(id);
		if (!informe.isPresent()) {
			flash.addFlashAttribute("error", "No existe el informe");
			return "redirect:/informe/listar";
		}

		model.put("informe", informe.get());
		model.put("title", "Detalle informe");
		return "/informe/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		String ruta = sistemaExpertoService.generar(model);
		
		Informe informe = new Informe();
		informe.setRuta(ruta);
		
		model.put("informe", informe);
		model.put("title", "Crear informe");
		return "/informe/form";
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Informe informe, BindingResult result, Model model, SessionStatus status) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Empleado");
			return "/informe/form";
		}
		
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		informe.setUsuario(usuarioService.findByUsername(authentication.getName()));
		
		informeService.save(informe);
		status.setComplete();
		return "redirect:/informe/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
		
		Optional<Informe> informe = informeService.findById(id);
		
		if(informe.isPresent()) {
			informeService.deleteById(id);
		}
		return "redirect:/informe/listar";
	}
}
