package uoc.tfg.g2tde.controllers;

import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Rol;
import uoc.tfg.g2tde.entities.Usuario;
import uoc.tfg.g2tde.interfaces.IRolService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/rol")
@SessionAttributes("rol")
@Secured({"ROLE_ADMIN"})
public class RolController {

	@Autowired
	private IRolService rolService;
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Roles");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Rol> roles = rolService.findAll(pageRequest);
		PageRender<Rol> pageRender = new PageRender<Rol>("/rol/listar", roles);
		model.addAttribute("roles", roles);
		model.addAttribute("page", pageRender);
		
		return "/rol/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Rol> rol = rolService.findById(id);
		if (!rol.isPresent()) {
			flash.addFlashAttribute("error", "No existe el rol");
			return "redirect:/rol/listar";
		}

		model.put("rol", rol.get());
		model.put("title", "Detalle rol");
		return "/rol/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		Rol rol = new Rol();
		model.put("rol", rol);
		model.put("title", "Crear rol");
		return "/rol/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		
		Optional<Rol> rolOpt = rolService.findById(id);
		
		if(rolOpt.isPresent()) {			
			model.put("rol", rolOpt.get());
			model.put("title", "Editar Rol");
			return "/rol/form";
		} else {
			return "redirect:/rol/listar";
		}
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Rol rol, BindingResult result, Model model, SessionStatus status) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Rol");
			return "/rol/form";
		}
		
		rolService.save(rol);
		status.setComplete();
		return "redirect:/rol/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
		if(id > 0) {
			rolService.findById(id).ifPresent(r->{
				for(Usuario usuario: r.getUsuarios()) {
					usuario.getRoles().remove(r);
				}
			});
			
			rolService.deleteById(id);
		}
		
		return "redirect:/rol/listar";
	}
}
