package uoc.tfg.g2tde.controllers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Cliente;
import uoc.tfg.g2tde.entities.Empleado;
import uoc.tfg.g2tde.entities.EmpleadoClienteTarea;
import uoc.tfg.g2tde.entities.Tarea;
import uoc.tfg.g2tde.interfaces.IClienteService;
import uoc.tfg.g2tde.interfaces.IEmpleadoClienteTareaService;
import uoc.tfg.g2tde.interfaces.IEmpleadoService;
import uoc.tfg.g2tde.interfaces.ITareaService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/tarea")
@SessionAttributes("tarea")
@Secured({"ROLE_GESTION"})
public class TareaController {

	@Autowired
	private ITareaService tareaService;
	
	@Autowired
	private IClienteService clienteService;
	
	@Autowired
	private IEmpleadoService empleadoService;
	
	@Autowired
	private IEmpleadoClienteTareaService empleadoClienteTareaService;
	
	@ModelAttribute("clientes")
	public List<Cliente> listaClientes(){
		return clienteService.findAll();
	}
	
	@ModelAttribute("empleados")
	public List<Empleado> listaEmpleados(){
		return empleadoService.findAll();
	}
	
	@ModelAttribute("tipos")
	public List<String> listaTipos(){
		List<String> tipos = new ArrayList<String>();
		tipos.add("Evolutivo");
		tipos.add("Correctivo");
		tipos.add("Incidencia");
		
		return tipos;
	}
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Tareas");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Tarea> tareas = tareaService.findAll(pageRequest);
		PageRender<Tarea> pageRender = new PageRender<Tarea>("/tarea/listar", tareas);
		model.addAttribute("title", "Listado de tareas");
		model.addAttribute("tareas", tareas);
		model.addAttribute("page", pageRender);
		
		return "/tarea/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {
		Optional<Tarea> tarea = tareaService.findById(id);
		if (!tarea.isPresent()) {
			flash.addFlashAttribute("error", "No existe la tarea");
			return "redirect:/tarea/listar";
		}

		model.put("tarea", tarea.get());
		model.put("title", "Detalle tarea");
		
		Iterator<EmpleadoClienteTarea> empleadoClienteTareaIterator = tarea.get().getEmpleadoClienteTarea().iterator();
		if(empleadoClienteTareaIterator.hasNext()) {
			EmpleadoClienteTarea empleadoClienteTarea = empleadoClienteTareaService.findById(
				empleadoClienteTareaIterator.next().getId()).get();
			
			model.put("empleado", empleadoClienteTarea.getEmpleado());
			model.put("cliente", empleadoClienteTarea.getCliente().getNombre());
		}
		
		return "/tarea/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {
		Tarea tarea = new Tarea();
		model.put("tarea", tarea);
		model.put("title", "Crear tarea");
		return "/tarea/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		Optional<Tarea> tareaOpt = tareaService.findById(id);		
		
		if(tareaOpt.isPresent()) {
			Tarea tarea = tareaOpt.get();
			
			Iterator<EmpleadoClienteTarea> empleadoClienteTareaIterator = tarea.getEmpleadoClienteTarea().iterator();
			if(empleadoClienteTareaIterator.hasNext()) {
				EmpleadoClienteTarea empleadoClienteTarea = empleadoClienteTareaService.findById(
					empleadoClienteTareaIterator.next().getId()).get();
				
				model.put("empleado", empleadoClienteTarea.getEmpleado().getId());
				model.put("cliente", empleadoClienteTarea.getCliente().getId());
			}
			
			model.put("tarea", tarea);
			model.put("title", "Editar Tarea");
			
			return "/tarea/form";
		} else {
			return "redirect:/tarea/listar";
		}
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Tarea tarea, BindingResult result, Model model, SessionStatus status, HttpServletRequest request) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Tarea");
			return "/tarea/form";
		}
		
		Tarea tareaSaved = tareaService.save(tarea);
		
		String clienteId = request.getParameter("cliente");
		String empleadoId = request.getParameter("empleado");
		
		if(clienteId != null && empleadoId != null) {
			EmpleadoClienteTarea empleadoClienteTarea = null;
			
			if(tarea.getEmpleadoClienteTarea() == null || tarea.getEmpleadoClienteTarea().isEmpty()) {
				empleadoClienteTarea = new EmpleadoClienteTarea();
				empleadoClienteTarea.setCliente(clienteService.findById(Long.parseLong(clienteId)).get());
				empleadoClienteTarea.setEmpleado(empleadoService.findById(Long.parseLong(empleadoId)).get());
				empleadoClienteTarea.setTarea(tareaSaved);
			}else {
				Iterator<EmpleadoClienteTarea> empleadoClienteTareaIterator = tarea.getEmpleadoClienteTarea().iterator();
				if(empleadoClienteTareaIterator.hasNext()) {
					empleadoClienteTarea = empleadoClienteTareaService.findById(
						empleadoClienteTareaIterator.next().getId()).get();
					
					if(empleadoClienteTarea.getCliente().getId() != Long.parseLong(clienteId)){
						empleadoClienteTarea.setCliente(clienteService.findById(Long.parseLong(clienteId)).get());
					}
					
					if(empleadoClienteTarea.getEmpleado().getId() != Long.parseLong(empleadoId)){
						empleadoClienteTarea.setEmpleado(empleadoService.findById(Long.parseLong(empleadoId)).get());
					}
				}
			}
			
			empleadoClienteTareaService.save(empleadoClienteTarea);
		}
		
		status.setComplete();
		return "redirect:/tarea/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
			
		Optional<Tarea> tareaOpt = tareaService.findById(id);
		
		if(tareaOpt.isPresent()) {			
			tareaService.deleteById(id);
		}
		
		return "redirect:/tarea/listar";
	}
}
