package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

@Entity
public class Cliente implements Serializable{
	
	private static final long serialVersionUID = -2320137294685363423L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "cliente_id")
	@SequenceGenerator(name = "cliente_id", sequenceName = "cliente_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 100, nullable = false)
	private String nombre;
	
	@Column(length = 150)
	private String direccion;
	
	@Pattern(regexp="(^$|[0-9]{9})", message = "No se corresponde con un formato de teléfono. Un formato de teléfono contiene 9 dígitos")
	@Column(length = 9)
	private String telefono;
	
	@NotEmpty
	@Email
	@Column(length = 60, nullable = false)
	private String email;
	
	@NotNull
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
    @JoinColumn(name = "consultoria_id", referencedColumnName = "id")
	private Consultoria consultoria;
	
	@OneToMany(mappedBy = "cliente", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<EmpleadoClienteTarea> empleadoClienteTarea;
	
	public Cliente() {
		super();
	}

	public Cliente(Long id, String nombre, String direccion, String telefono, String email,
		Consultoria consultoria) {
		super();
		this.id = id;
		this.nombre = nombre;
		this.direccion = direccion;
		this.telefono = telefono;
		this.email = email;
		this.consultoria = consultoria;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDireccion() {
		return direccion;
	}

	public void setDireccion(String direccion) {
		this.direccion = direccion;
	}

	public String getTelefono() {
		return telefono;
	}

	public void setTelefono(String telefono) {
		this.telefono = telefono;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Consultoria getConsultoria() {
		return consultoria;
	}

	public void setConsultoria(Consultoria consultoria) {
		this.consultoria = consultoria;
	}

	public Set<EmpleadoClienteTarea> getEmpleadoClienteTarea() {
		return empleadoClienteTarea;
	}

	public void setEmpleadoClienteTarea(Set<EmpleadoClienteTarea> empleadoClienteTarea) {
		this.empleadoClienteTarea = empleadoClienteTarea;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((direccion == null) ? 0 : direccion.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((telefono == null) ? 0 : telefono.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Cliente other = (Cliente) obj;
		if (direccion == null) {
			if (other.direccion != null)
				return false;
		} else if (!direccion.equals(other.direccion))
			return false;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (telefono == null) {
			if (other.telefono != null)
				return false;
		} else if (!telefono.equals(other.telefono))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Consultoria [id=" + id + ", nombre=" + nombre + ", direccion=" + direccion + ", telefono=" + telefono
				+ ", email=" + email + "]";
	}
}
