package uoc.tfg.g2tde.entities;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.PreRemove;
import javax.persistence.SequenceGenerator;
import javax.validation.constraints.NotEmpty;

@Entity
public class Conclusion implements Serializable{
	
	private static final long serialVersionUID = -3179608938393212332L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "conclusion_id")
	@SequenceGenerator(name = "conclusion_id", sequenceName = "conclusion_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 60, nullable = false)
	private String nombre;
	
	@Column(length = 100)
	private String descripcion;
	
	@Column(name = "es_calculada", length = 1, nullable = false)
	private String esCalculada;
	
	@Column(length = 1, nullable = false)
	private String activa;
	
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
    @JoinColumn(name = "usuario_id", referencedColumnName = "username")
	private Usuario usuario;
	
	public Conclusion() {
		super();
	}
	
	public Conclusion(Long id, String nombre, String descripcion, String tipo, String esCalculada, String activa, Usuario usuario) {
		super();
		this.id = id;
		this.nombre = nombre;
		this.descripcion = descripcion;
		this.esCalculada = esCalculada;
		this.activa = activa;
		this.usuario = usuario;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public String getEsCalculada() {
		return esCalculada;
	}

	public void setEsCalculada(String esCalculada) {
		this.esCalculada = esCalculada;
	}

	public String getActiva() {
		return activa;
	}

	public void setActiva(String activa) {
		this.activa = activa;
	}

	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}
	
	@PreRemove
	public void preRemove() {
		this.setUsuario(null);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activa == null) ? 0 : activa.hashCode());
		result = prime * result + ((descripcion == null) ? 0 : descripcion.hashCode());
		result = prime * result + ((esCalculada == null) ? 0 : esCalculada.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((usuario == null) ? 0 : usuario.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Conclusion other = (Conclusion) obj;
		if (activa == null) {
			if (other.activa != null)
				return false;
		} else if (!activa.equals(other.activa))
			return false;
		if (descripcion == null) {
			if (other.descripcion != null)
				return false;
		} else if (!descripcion.equals(other.descripcion))
			return false;
		if (esCalculada == null) {
			if (other.esCalculada != null)
				return false;
		} else if (!esCalculada.equals(other.esCalculada))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (usuario == null) {
			if (other.usuario != null)
				return false;
		} else if (!usuario.equals(other.usuario))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Conclusion [id=" + id + ", nombre=" + nombre + ", descripcion=" + descripcion + ", esCalculada="
				+ esCalculada + ", activa=" + activa + ", usuario=" + usuario + "]";
	}
}
