package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.SequenceGenerator;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

import org.springframework.format.annotation.DateTimeFormat;

@Entity
public class Criterio implements Serializable{
	
	private static final long serialVersionUID = -7067755340753883802L;
	
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "criterio_id")
	@SequenceGenerator(name = "criterio_id", sequenceName = "criterio_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 60, nullable = false)
	private String nombre;
	
	@Column(length = 100)
	private String descripcion;
	
	@Column(name = "fecha_alta")
	@Temporal(TemporalType.DATE)
	@DateTimeFormat(pattern="yyyy-MM-dd")
	private Date fechaAlta;
	
	@NotEmpty
	@Column(length = 20, nullable = false)
	private String tipo;
	
	@NotEmpty
	@Column(length = 20, nullable = false)
	private String subtipo;
	
	@NotNull
	@Column(length = 10, precision=2, nullable = false)
	private BigDecimal valor;
	
	@Column(name = "es_calculado", length = 1, nullable = false)
	private String esCalculado;
	
	@NotEmpty
	@Column(length = 1, nullable = false)
	private String activo;
	
	@OneToOne
    @JoinColumn(name = "usuario_id", referencedColumnName = "username")
	private Usuario usuario;
	
	@NotNull
	@OneToOne
    @JoinColumn(name = "operador_id", referencedColumnName = "id")
	private Operador operador;
	
	@OneToMany(mappedBy = "criterio", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<ReglaCriterioOperador> reglaCriterioOperadores;
	
	public Criterio() {}

	public Criterio(Long id, String nombre, String descripcion, Date fechaAlta, String tipo, String subtipo, String activo,
			Usuario usuario) {
		super();
		this.id = id;
		this.nombre = nombre;
		this.descripcion = descripcion;
		this.fechaAlta = fechaAlta;
		this.tipo = tipo;
		this.subtipo = subtipo;
		this.activo = activo;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public Date getFechaAlta() {
		return fechaAlta;
	}

	public void setFechaAlta(Date fechaAlta) {
		this.fechaAlta = fechaAlta;
	}

	public String getTipo() {
		return tipo;
	}

	public void setTipo(String tipo) {
		this.tipo = tipo;
	}

	public String getSubtipo() {
		return subtipo;
	}

	public void setSubtipo(String subtipo) {
		this.subtipo = subtipo;
	}

	public BigDecimal getValor() {
		return valor;
	}

	public void setValor(BigDecimal valor) {
		this.valor = valor;
	}

	public String getEsCalculado() {
		return esCalculado;
	}

	public void setEsCalculado(String esCalculado) {
		this.esCalculado = esCalculado;
	}

	public String getActivo() {
		return activo;
	}

	public void setActivo(String activo) {
		this.activo = activo;
	}
	
	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	public Operador getOperador() {
		return operador;
	}

	public void setOperador(Operador operador) {
		this.operador = operador;
	}

	public Set<ReglaCriterioOperador> getReglaCriterioOperadores() {
		return reglaCriterioOperadores;
	}

	public void setReglaCriterioOperadores(Set<ReglaCriterioOperador> reglaCriterioOperadores) {
		this.reglaCriterioOperadores = reglaCriterioOperadores;
	}
	
	@PrePersist
	public void prePersist() {
		if(this.fechaAlta == null) {
			fechaAlta = new Date();
		}
	}
	
	@PreRemove
	public void preRemove() {
		this.setUsuario(null);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activo == null) ? 0 : activo.hashCode());
		result = prime * result + ((descripcion == null) ? 0 : descripcion.hashCode());
		result = prime * result + ((esCalculado == null) ? 0 : esCalculado.hashCode());
		result = prime * result + ((fechaAlta == null) ? 0 : fechaAlta.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((subtipo == null) ? 0 : subtipo.hashCode());
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		result = prime * result + ((valor == null) ? 0 : valor.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Criterio other = (Criterio) obj;
		if (activo == null) {
			if (other.activo != null)
				return false;
		} else if (!activo.equals(other.activo))
			return false;
		if (descripcion == null) {
			if (other.descripcion != null)
				return false;
		} else if (!descripcion.equals(other.descripcion))
			return false;
		if (esCalculado == null) {
			if (other.esCalculado != null)
				return false;
		} else if (!esCalculado.equals(other.esCalculado))
			return false;
		if (fechaAlta == null) {
			if (other.fechaAlta != null)
				return false;
		} else if (!fechaAlta.equals(other.fechaAlta))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (subtipo == null) {
			if (other.subtipo != null)
				return false;
		} else if (!subtipo.equals(other.subtipo))
			return false;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		if (valor == null) {
			if (other.valor != null)
				return false;
		} else if (!valor.equals(other.valor))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Criterio [id=" + id + ", nombre=" + nombre + ", descripcion=" + descripcion + ", fechaAlta=" + fechaAlta
				+ ", tipo=" + tipo + ", subtipo=" + subtipo + ", valor=" + valor + ", esCalculado=" + esCalculado
				+ ", activo=" + activo + "]";
	}	
}
