package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import org.springframework.format.annotation.DateTimeFormat;

@Entity
public class Empleado implements Serializable{
	
	private static final long serialVersionUID = 3439212886337817019L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "empleado_id")
	@SequenceGenerator(name = "empleado_id", sequenceName = "empleado_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 150, nullable = false)
	private String nombre;
	
	@NotEmpty
	@Column(length = 100, nullable = false)
	private String apellido1;
	
	@NotEmpty
	@Column(length = 100, nullable = false)
	private String apellido2;
	
	@Pattern(regexp="(^$|[0-9]{9})", message = "No se corresponde con un formato de teléfono. Un formato de teléfono contiene 9 dígitos")
	@Column(length = 9)
	private String telefono;
	
	@NotEmpty
	@Email
	@Column(length = 60, nullable = false)
	private String email;
	
	@NotNull
	@Column(name = "fecha_contratacion")
	@Temporal(TemporalType.DATE)
	@DateTimeFormat(pattern="yyyy-MM-dd")
	private Date fechaContratacion;
	
	@NotNull
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
    @JoinColumn(name = "consultoria_id", referencedColumnName = "id")
	private Consultoria consultoria;
	
	@OneToOne(mappedBy="empleado", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
	private Usuario usuario;
	
	@OneToMany(mappedBy = "empleado", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<EmpleadoClienteTarea> empleadoClienteTarea;
	
	public Empleado() {}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getApellido1() {
		return apellido1;
	}

	public void setApellido1(String apellido1) {
		this.apellido1 = apellido1;
	}

	public String getApellido2() {
		return apellido2;
	}

	public void setApellido2(String apellido2) {
		this.apellido2 = apellido2;
	}

	public String getTelefono() {
		return telefono;
	}

	public void setTelefono(String telefono) {
		this.telefono = telefono;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Date getFechaContratacion() {
		return fechaContratacion;
	}

	public void setFechaContratacion(Date fechaContratacion) {
		this.fechaContratacion = fechaContratacion;
	}

	public Consultoria getConsultoria() {
		return consultoria;
	}

	public void setConsultoria(Consultoria consultoria) {
		this.consultoria = consultoria;
	}

	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	public Set<EmpleadoClienteTarea> getEmpleadoClienteTarea() {
		return empleadoClienteTarea;
	}

	public void setEmpleadoClienteTarea(Set<EmpleadoClienteTarea> empleadoClienteTarea) {
		this.empleadoClienteTarea = empleadoClienteTarea;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((apellido1 == null) ? 0 : apellido1.hashCode());
		result = prime * result + ((apellido2 == null) ? 0 : apellido2.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((fechaContratacion == null) ? 0 : fechaContratacion.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((telefono == null) ? 0 : telefono.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Empleado other = (Empleado) obj;
		if (apellido1 == null) {
			if (other.apellido1 != null)
				return false;
		} else if (!apellido1.equals(other.apellido1))
			return false;
		if (apellido2 == null) {
			if (other.apellido2 != null)
				return false;
		} else if (!apellido2.equals(other.apellido2))
			return false;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		if (fechaContratacion == null) {
			if (other.fechaContratacion != null)
				return false;
		} else if (!fechaContratacion.equals(other.fechaContratacion))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (telefono == null) {
			if (other.telefono != null)
				return false;
		} else if (!telefono.equals(other.telefono))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Empleado [id=" + id + ", nombre=" + nombre + ", apellido1=" + apellido1 + ", apellido2=" + apellido2
				+ ", telefono=" + telefono + ", email=" + email + ", fechaContratacion=" + fechaContratacion + "]";
	}
}
