package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.validation.constraints.NotEmpty;

@Entity
public class Operador implements Serializable{
	
	private static final long serialVersionUID = -3166391464519405600L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "operador_id")
	@SequenceGenerator(name = "operador_id", sequenceName = "operador_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 20, nullable = false)
	private String nombre;
	
	@NotEmpty
	@Column(length = 20, nullable = false)
	private String tipo;
	
	@NotEmpty
	@Column(length = 20, nullable = false)
	private String componente;
	
	@OneToMany(mappedBy = "regla", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<ReglaCriterioOperador> reglaCriterioOperadores;
	
	public Operador() {}

	public Operador(Long id, @NotEmpty String nombre, @NotEmpty String tipo, @NotEmpty String componente) {
		this.id = id;
		this.nombre = nombre;
		this.tipo = tipo;
		this.componente = componente;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	
	public String getTipo() {
		return tipo;
	}

	public void setTipo(String tipo) {
		this.tipo = tipo;
	}

	public String getComponente() {
		return componente;
	}

	public void setComponente(String componente) {
		this.componente = componente;
	}

	public Set<ReglaCriterioOperador> getReglaCriterioOperadores() {
		return reglaCriterioOperadores;
	}

	public void setReglaCriterioOperadores(Set<ReglaCriterioOperador> reglaCriterioOperadores) {
		this.reglaCriterioOperadores = reglaCriterioOperadores;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((componente == null) ? 0 : componente.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((reglaCriterioOperadores == null) ? 0 : reglaCriterioOperadores.hashCode());
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Operador other = (Operador) obj;
		if (componente == null) {
			if (other.componente != null)
				return false;
		} else if (!componente.equals(other.componente))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (reglaCriterioOperadores == null) {
			if (other.reglaCriterioOperadores != null)
				return false;
		} else if (!reglaCriterioOperadores.equals(other.reglaCriterioOperadores))
			return false;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Operador [id=" + id + ", nombre=" + nombre + ", tipo=" + tipo + ", componente=" + componente
				+ ", reglaCriterioOperadores=" + reglaCriterioOperadores + "]";
	}
}
