package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PrePersist;
import javax.persistence.SequenceGenerator;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

import org.springframework.format.annotation.DateTimeFormat;

@Entity
public class Regla implements Serializable{
	
	private static final long serialVersionUID = -9044315651892123919L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "regla_id")
	@SequenceGenerator(name = "regla_id", sequenceName = "regla_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 60, nullable = false)
	private String nombre;
	
	@Column(length = 100)
	private String descripcion;
	
	@Column(name = "fecha_alta")
	@Temporal(TemporalType.DATE)
	@DateTimeFormat(pattern="dd-MM-yyyy")
	private Date fechaAlta;
	
	@Column(name = "es_calculada", length = 1, nullable = false)
	private String esCalculada;
	
	@NotEmpty
	@Column(length = 1, nullable = false)
	private String activa;
	
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE})
    @JoinColumn(name = "usuario_id", referencedColumnName = "username")
	private Usuario usuario;
	
	@NotNull
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
	@JoinColumn(name="conclusion_id")
	private Conclusion conclusion;
	
	@OneToMany(mappedBy = "regla", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<ReglaCriterioOperador> reglaCriterioOperadores;
	
	public Regla() {}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public Date getFechaAlta() {
		return fechaAlta;
	}

	public void setFechaAlta(Date fechaAlta) {
		this.fechaAlta = fechaAlta;
	}

	public String getEsCalculada() {
		return esCalculada;
	}

	public void setEsCalculada(String esCalculada) {
		this.esCalculada = esCalculada;
	}

	public String getActiva() {
		return activa;
	}

	public void setActiva(String activa) {
		this.activa = activa;
	}

	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	public Conclusion getConclusion() {
		return conclusion;
	}

	public void setConclusion(Conclusion conclusion) {
		this.conclusion = conclusion;
	}

	public Set<ReglaCriterioOperador> getReglaCriterioOperadores() {
		return reglaCriterioOperadores;
	}

	public void setReglaCriterioOperadores(Set<ReglaCriterioOperador> reglaCriterioOperadores) {
		this.reglaCriterioOperadores = reglaCriterioOperadores;
	}
	
	@PrePersist
	public void prePersist() {
		if(this.fechaAlta == null) {
			fechaAlta = new Date();
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activa == null) ? 0 : activa.hashCode());
		result = prime * result + ((descripcion == null) ? 0 : descripcion.hashCode());
		result = prime * result + ((esCalculada == null) ? 0 : esCalculada.hashCode());
		result = prime * result + ((fechaAlta == null) ? 0 : fechaAlta.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Regla other = (Regla) obj;
		if (activa == null) {
			if (other.activa != null)
				return false;
		} else if (!activa.equals(other.activa))
			return false;
		if (descripcion == null) {
			if (other.descripcion != null)
				return false;
		} else if (!descripcion.equals(other.descripcion))
			return false;
		if (esCalculada == null) {
			if (other.esCalculada != null)
				return false;
		} else if (!esCalculada.equals(other.esCalculada))
			return false;
		if (fechaAlta == null) {
			if (other.fechaAlta != null)
				return false;
		} else if (!fechaAlta.equals(other.fechaAlta))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Regla [id=" + id + ", nombre=" + nombre + ", descripcion=" + descripcion + ", fechaAlta=" + fechaAlta
				+ ", esCalculada=" + esCalculada + ", activa=" + activa + "]";
	}
}
