package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.PositiveOrZero;

@Entity
public class Tarea implements Serializable{
	
	private static final long serialVersionUID = 2553593536552870240L;

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "tarea_id")
	@SequenceGenerator(name = "tarea_id", sequenceName = "tarea_id", allocationSize = 1, initialValue = 1)
	private Long id;
	
	@NotEmpty
	@Column(length = 60, nullable = false)
	private String nombre;
	
	@NotEmpty
	@Column(length = 100, nullable = false)
	private String descripcion;
	
	@NotEmpty
	@Column(length = 10, nullable = false)
	private String tipo;
	
	@NotNull
	@PositiveOrZero
	@Column(length = 10, precision = 2 , nullable = false)
	private BigDecimal estimado;
	
	@NotNull
	@PositiveOrZero
	@Column(length = 10, precision = 2 , nullable = false)
	private BigDecimal incurrido;
	
	@OneToMany(mappedBy = "tarea", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    private Set<EmpleadoClienteTarea> empleadoClienteTarea;
	
	public Tarea() {
		super();
	}

	public Tarea(Long id, String nombre, String descripcion, String tipo, BigDecimal estimado, BigDecimal incurrido) {
		super();
		this.id = id;
		this.nombre = nombre;
		this.descripcion = descripcion;
		this.tipo = tipo;
		this.estimado = estimado;
		this.incurrido = incurrido;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public String getTipo() {
		return tipo;
	}

	public void setTipo(String tipo) {
		this.tipo = tipo;
	}

	public BigDecimal getEstimado() {
		return estimado;
	}

	public void setEstimado(BigDecimal estimado) {
		this.estimado = estimado;
	}

	public BigDecimal getIncurrido() {
		return incurrido;
	}

	public void setIncurrido(BigDecimal incurrido) {
		this.incurrido = incurrido;
	}

	public Set<EmpleadoClienteTarea> getEmpleadoClienteTarea() {
		return empleadoClienteTarea;
	}

	public void setEmpleadoClienteTarea(Set<EmpleadoClienteTarea> empleadoClienteTarea) {
		this.empleadoClienteTarea = empleadoClienteTarea;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((descripcion == null) ? 0 : descripcion.hashCode());
		result = prime * result + ((estimado == null) ? 0 : estimado.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((incurrido == null) ? 0 : incurrido.hashCode());
		result = prime * result + ((nombre == null) ? 0 : nombre.hashCode());
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Tarea other = (Tarea) obj;
		if (descripcion == null) {
			if (other.descripcion != null)
				return false;
		} else if (!descripcion.equals(other.descripcion))
			return false;
		if (estimado == null) {
			if (other.estimado != null)
				return false;
		} else if (!estimado.equals(other.estimado))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (incurrido == null) {
			if (other.incurrido != null)
				return false;
		} else if (!incurrido.equals(other.incurrido))
			return false;
		if (nombre == null) {
			if (other.nombre != null)
				return false;
		} else if (!nombre.equals(other.nombre))
			return false;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Tarea [id=" + id + ", nombre=" + nombre + ", descripcion=" + descripcion + ", tipo=" + tipo
				+ ", estimado=" + estimado + ", incurrido=" + incurrido + "]";
	}
}
