package uoc.tfg.g2tde.entities;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PreRemove;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

@Entity
public class Usuario implements Serializable{
	
	private static final long serialVersionUID = -3166391464519405600L;

	@Id
	@NotEmpty
	@Column(length = 20)
	private String username;
	
	@NotEmpty
	@Column(length = 200, nullable = false)
	private String password;
	
	@OneToOne(cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
	@JoinColumn(name="empleado_id")
	private Empleado empleado;
	
	@NotNull
	@ManyToMany(fetch = FetchType.EAGER, cascade = {CascadeType.PERSIST, CascadeType.MERGE})
	@JoinTable(name = "rol_usuario",
    	joinColumns = {@JoinColumn(name = "usuario_username")},
    	inverseJoinColumns = { @JoinColumn(name = "rol_id")}
	)
    private Set<Rol> roles;
	
	@OneToMany(mappedBy = "usuario", cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
    private Set<Regla> reglas;
	
	@OneToMany(mappedBy = "usuario", cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
    private Set<Conclusion> conclusiones;
	
	@OneToMany(mappedBy = "usuario", cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
    private Set<Criterio> criterios;
	
	@OneToMany(mappedBy = "usuario", cascade = {CascadeType.PERSIST, CascadeType.MERGE}, fetch = FetchType.LAZY)
    private Set<Informe> informes;
	
	public Usuario() {}

	public Usuario(String username, String password) {
		this.username = username;
		this.password = password;
	}

	public Usuario(String username, @NotEmpty String password, Set<Rol> roles) {
		this.username = username;
		this.password = password;
		this.roles = roles;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}
	
	public Empleado getEmpleado() {
		return empleado;
	}

	public void setEmpleado(Empleado empleado) {
		this.empleado = empleado;
	}

	public Set<Rol> getRoles() {
		return roles;
	}

	public void setRoles(Set<Rol> roles) {
		this.roles = roles;
	}

	public Set<Regla> getReglas() {
		return reglas;
	}

	public void setReglas(Set<Regla> reglas) {
		this.reglas = reglas;
	}

	public Set<Conclusion> getConclusiones() {
		return conclusiones;
	}

	public void setConclusiones(Set<Conclusion> conclusiones) {
		this.conclusiones = conclusiones;
	}

	public Set<Criterio> getCriterios() {
		return criterios;
	}

	public void setCriterios(Set<Criterio> criterios) {
		this.criterios = criterios;
	}
	
	public Set<Informe> getInformes() {
		return informes;
	}

	public void setInformes(Set<Informe> informes) {
		this.informes = informes;
	}

	@PreRemove
	public void preRemove() {
		this.setEmpleado(null);
		this.conclusiones.stream().forEach(c-> c.setUsuario(null));
		this.criterios.stream().forEach(c-> c.setUsuario(null));
		this.reglas.stream().forEach(r-> r.setUsuario(null));
		this.informes.stream().forEach(i-> i.setUsuario(null));
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((password == null) ? 0 : password.hashCode());
		result = prime * result + ((username == null) ? 0 : username.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Usuario other = (Usuario) obj;
		if (password == null) {
			if (other.password != null)
				return false;
		} else if (!password.equals(other.password))
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Usuario [username=" + username + ", password=" + password + "]";
	}		
}
