package uoc.tfg.g2tde.ssee.services;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import uoc.tfg.g2tde.entities.Conclusion;
import uoc.tfg.g2tde.entities.Criterio;
import uoc.tfg.g2tde.entities.Regla;
import uoc.tfg.g2tde.entities.ReglaCriterioOperador;
import uoc.tfg.g2tde.entities.Tarea;
import uoc.tfg.g2tde.interfaces.IConclusionService;
import uoc.tfg.g2tde.interfaces.IReglaService;
import uoc.tfg.g2tde.interfaces.ITareaService;
import uoc.tfg.g2tde.ssee.interfaces.ISistemaExpertoService;

@Service
public class SistemaExpertoService implements ISistemaExpertoService{
	
	@Autowired
	private IReglaService reglaService;
	
	@Autowired
	private IConclusionService conclusionService;
	
	@Autowired
	private ITareaService tareaService;
	
	Set<Long> ts = new TreeSet<>();
	
	@Override
	public String generar(Map<String, Object> model) {
		
		StringBuilder sb = new StringBuilder(); 
		
		List<Regla> reglaList = reglaService.findAll().stream()
			.filter(r->"S".equalsIgnoreCase(r.getActiva()))
			.collect(Collectors.toList());
		
		for(Regla regla: reglaList) {
			getTree(regla);
		}
		
		System.out.println("El árbol resultante es: ");
		
		ts.stream().forEach(System.out::println);
		ts.stream().forEach(t->{			
			Optional<Conclusion> conclusionOpt = conclusionService.findById(t);
			
			System.out.println("Conclusión: "+ conclusionOpt);
			
			if(conclusionOpt.isPresent()) {
				sb.append(conclusionOpt.get().getNombre());
				sb.append(", ");
			}
		});
		
		return sb.toString();
	}	
	
	private void getTree(Regla regla) {
		List<Boolean> isReglaCompuestaValid = new ArrayList<>();
		Boolean isValid = Boolean.FALSE;
		Boolean isReglaCompuesta= Boolean.FALSE;
		
		if(regla.getReglaCriterioOperadores() != null) {
			int i = 0;
			
			for(ReglaCriterioOperador r: regla.getReglaCriterioOperadores()){
				isReglaCompuesta = "AND".equalsIgnoreCase(r.getOperador().getNombre()) || "OR".equalsIgnoreCase(r.getOperador().getNombre());
				
				if("S".equalsIgnoreCase(r.getCriterio().getActivo())) {
					switch(r.getCriterio().getTipo()) {
						case "Empleado":
							// fecha de contratación y nombre (para hacer combinaciones)
							System.out.println("Empleado");
							if("Fecha de contratación".equalsIgnoreCase(r.getCriterio().getSubtipo())) {
								
							}
							break;
						case "Tarea":
							isValid = checkSubtype(r.getCriterio(), tareaService.findAll());
							break;
					}
					
					if("NOT".equalsIgnoreCase(r.getOperador().getNombre())) {
						isValid = !isValid;
					}
					
					if(i == 0) {
						isReglaCompuestaValid.add(isValid);
					}else if(i == 1) {
						isReglaCompuestaValid.add(isValid);
					}
				}
			}
			
			if(!isReglaCompuestaValid.isEmpty()) {
				if((Boolean.TRUE.equals(isReglaCompuestaValid.get(0)) && Boolean.FALSE.equals(isReglaCompuesta))
					|| (Boolean.TRUE.equals(isReglaCompuestaValid.get(0)) && Boolean.TRUE.equals(isReglaCompuestaValid.get(1)) 
						&& Boolean.TRUE.equals(isReglaCompuesta))) {
					if("S".equalsIgnoreCase(regla.getConclusion().getActiva())) {
						ts.add(regla.getConclusion().getId());
					}
				}
			}
		}
	}
	
	private Boolean checkSubtype(Criterio criterio, List<Tarea> tareaList) {
		Boolean isValid = Boolean.FALSE;
		
		switch(criterio.getSubtipo()) {
			case "Tipo":
				tareaList.stream().collect(Collectors.groupingBy(Tarea::getTipo));	
				break;
			case "Estimado":
				BigDecimal estimadoTotal = tareaList.stream()
					.map(t->t.getEstimado())
					.reduce(BigDecimal.ZERO,BigDecimal::add);
				isValid = checkOperatorWithCriteria(criterio, estimadoTotal);
				break;
			case "Incurrido":
				BigDecimal incurridoTotal = tareaList.stream()
					.map(t->t.getIncurrido())
					.reduce(BigDecimal.ZERO,BigDecimal::add);
				isValid = checkOperatorWithCriteria(criterio, incurridoTotal);
				break;
			case "Cliente":
				break;
			case "Empleado":
				break;
		}
		
		return isValid;
	}
	

	private Boolean checkOperatorWithCriteria(Criterio criterio, BigDecimal valor) {
		Boolean isValid = Boolean.FALSE;
		
		switch(criterio.getOperador().getNombre()) {
			case "=":
				if(valor.compareTo(criterio.getValor()) == 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
			case "<":
				if(valor.compareTo(criterio.getValor()) < 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
			case "<=":
				if((valor.compareTo(criterio.getValor()) < 0) || valor.compareTo(criterio.getValor()) == 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
			case ">":
				if(valor.compareTo(criterio.getValor()) > 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
			case ">=":
				if((valor.compareTo(criterio.getValor()) > 0) || valor.compareTo(criterio.getValor()) == 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
			case "!=":
				if(valor.compareTo(criterio.getValor()) != 0) {
					System.out.println("Criterio válido");
					isValid = Boolean.TRUE;
				}
				break;
		}
		return isValid;
	}
}
